/**
 * @file
 * Attaches behavior for the Filter Tooltips module.
 */

(function ($, Drupal) {
  'use strict';

  /**
   * Displays the tooltips of the selected text format automatically.
   *
   * @type {Drupal~behavior}
   *
   * @prop {Drupal~behaviorAttach} attach
   *   Attaches behavior for updating filter tooltips.
   */
  Drupal.behaviors.filterTooltips = {
    attach: function attach(context, settings) {
      $('.filter-tooltips', context).on('click', function (event) {
        event.preventDefault();
        if (!$(this).next().hasClass('filter-tooltips-explanation')) {
          var div = jQuery('<div/>', {
            class: 'filter-tooltips-explanation',
            html: $(this).data('explanation')
          });
          $(this).after(div);
        }

        var explanation = $(this).next();
        if (explanation.hasClass('open')) {
          explanation.slideToggle(function () {
            explanation.remove();
          });
        }
        else {
          explanation.hide().addClass('open').slideToggle();
        }
      });
    }
  };
})(jQuery, Drupal);
